/*
Daybo Logic Run-Time TarBall Library And Tools
Copyright (c) 2000-2006, David Duncan Ross Palmer, Daybo Logic
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
      
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
      
    * Neither the name of the Daybo Logic nor the names of its contributors
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

/* Example program for using librttb, this example reads in every other
character from the README document and writes out a nonsense file called
README.2 which, as it contains every other character, should be unreadable.
README should be in a file called readme.rttb in the directory the example
is run from.  readme.rttb may contain whatever other rubbish you want,
as long as README is in there somewhere */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif /*HAVE_CONFIG_H*/

#include <stdio.h>

#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#endif /*HAVE_STDLIB_H*/

#ifdef HAVE_LIMITS_H
# include <limits.h>
#endif /*HAVE_LIMITS_H*/

#ifdef RTTB_HDRSTOP
# pragma hdrstop
#endif /*RTTB_HDRSTOP*/

#include "rttb.h"
/*------------------------------------------------------------*/
static void PrintError(RTTB_ERROR ErrorNumber);
static RTTB_ERROR TestLoop(RTTB_HANDLE InputHandle, FILE* OutputHandle);
static void LibStartup(void);

int main()
{
  RTTB_HANDLE handle;
  RTTB_ERROR lastError;

  LibStartup();
  handle = rttb_fopen("readme.rttb", "README", "rt", 0, &lastError);
  if ( handle ) {
    FILE* output = fopen("README.2", "wt");
    if ( output ) {
      lastError = TestLoop(handle, output);
      if ( lastError )
        PrintError(lastError);
      fclose(output);
    }
    rttb_fclose(handle);
  }
  else {
    PrintError(lastError);
  }
  return EXIT_SUCCESS;
}

static void PrintError(RTTB_ERROR ErrorNumber)
{
  char* msgcopy;
  unsigned int msglen;

  msglen = rttb_GetErrorMessage(ErrorNumber, NULL, 0U);
  msgcopy = (char*)malloc(msglen + 1);
  if ( msgcopy ) {
    rttb_GetErrorMessage(ErrorNumber, msgcopy, msglen);
    printf("librttb error: %s\n", msgcopy);
    free(msgcopy);
  }
  return;
}

static RTTB_ERROR TestLoop(RTTB_HANDLE InputHandle, FILE* OutputHandle)
{
  /* NOTE: There may be more efficient ways of donig this, there are, I
  know.  The point is not to be mega efficient but rather, to demonstrate
  usage of the most RTTB library functions. */

  RTTB_ERROR error = RTTB_ERROR_NOERROR;

  while ( !rttb_feof(InputHandle) ) {
    long int pos;
    unsigned char abyte;

    error = rttb_ftell(InputHandle, &pos);
    if ( error ) break;
    abyte = (unsigned char)rttb_fgetc(InputHandle, &error);
    if ( error ) break;
    if ( pos % 2 ) {
      if ( fputc((int)abyte, OutputHandle ) == EOF ) {
        printf("Out of disk or quota!\n");
        break;
      }
      else { /* Odd byte, seek to next even */
        error = rttb_fseek(InputHandle, 1, SEEK_CUR); /* Seek forward by one */
        if ( error ) break;
      }
    }
  }
  return error;
}

static void LibStartup()
{
  if ( rttb_Setup() )
    atexit(rttb_Cleanup);
  else
    abort();
}
