/*
Daybo Logic Run-Time TarBall Library And Tools
Copyright (c) 2000-2006, David Duncan Ross Palmer, Daybo Logic
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
      
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
      
    * Neither the name of the Daybo Logic nor the names of its contributors
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/



/* Main module of rttb interface library.  You must route all file requests
through this library in order for the read only rttb reader to work.
Here is a very brief description, for more information read the documents
in the rttb section of the docs branch of the Colelus source tree.

Brief: Instead of having a ton of files, pack the together with the IDW
utility makerttb.  Then access them in your code through this library.
All fread(), fseek() requests should go through this library so that file
positions are relative, also if a file exists outside of the rttb file it
is used in preference of the one in the rttb (although this behaviour can
be altered).

WARNING: Do not mix and match FILE* accepted and used by normal fopen()
calls with the special handles returned from these functions.

 - DDRP - www.daybologic.co.uk/mailddrp, author and maintainer.

21st October 2000
*/

#define RTTB_SOURCE

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif /*HAVE_CONFIG_H*/

#include <stdio.h>

#ifdef HAVE_STRING_H
# include <string.h>
#endif /*HAVE_STRING_H*/

#ifdef HAVE_LIMITS_H
# include <limits.h>
#endif /*HAVE_LIMITS_H*/

#ifdef RTTB_HDRSTOP
# pragma hdrstop
#endif /*RTTB_HDRSTOP*/

#include "rttb.h" /* Public library header */
#include "rttb_int.h" /* Internals */
#include "rttb_ll.h" /* Linked list */
#include "rttb_ver.h" /* Library version number */
/*--------------------------------------------------------------------------*/
int rttb_Setup()
{
  if (!rttb_StartedCheck())
  {
    rttb_core()->Init();
    #ifndef NDEBUG
    if ( !rttb_FileManagerStartup() ) { rttb_Cleanup(); return 0; }
    #endif /*!NDEBUG*/
  }
  return rttb_StartedCheck();
}
/*--------------------------------------------------------------------------*/
void rttb_Cleanup()
{
  if (rttb_StartedCheck())
  {
    #ifndef NDEBUG
    unsigned long blkcnt;
    #endif
    rttb_core()->Uninit();
    #ifndef NDEBUG
    if ( rttb_FileManagerIsStarted() ) rttb_FileManagerShutdown();
    blkcnt = rttb_DebugBlocks();
    if (blkcnt)
      fprintf(RTTB_DEV_ERROR, "Warning librttb leaked %lu blocks of memory\n", blkcnt);
    #endif /*!NDEBUG*/
  }
}
/*--------------------------------------------------------------------------*/
int rttb_StartedCheck()
{
  return rttb_core()->IsStarted();
}
/*-------------------------------------------------------------------------*/
unsigned int rttb_GetErrorMessage(const RTTB_ERROR ErrorNumber, char* PYourString, const unsigned int MaxLen)
{
  const char* msgptr;
  static const char defaultMsg[] = "Invalid and unknown error";
  static const char* msgs[] = {
    "No error has occurred.", /*RTTB_ERROR_NOERROR*/
    "A real file could not be opened", /*RTTB_ERROR_CANTOPEN*/
    "Attempt to open a file to write to through librttb, librttb is readonly", /*RTTB_ERROR_READONLY*/
    "The rttb handle was invalid", /*RTTB_ERROR_INVHANDLE*/
    "Seek past end of virtual file (or physical end of archive)", /*RTTB_ERROR_OVERSEEK*/
    "Seek before beginning of virtual file (or physical beginning of archive)", /*RTTB_ERROR_UNDERSEEK*/
    "Invalid arguments; maluse of library, link with debugging library to help track it down", /*RTTB_ERROR_INVPARAMS*/
    "Too many open rttb containers", /*RTTB_ERROR_OPENLIMIT*/
    "The container is too old and can\'t be supported, resync everything, rebuild all!", /*RTTB_ERROR_OLD*/
    "Don\'t know how to understand newer rttb container formats, resync everything, rebuild all!", /*RTTB_ERROR_NEW*/
    "Unexpected end of archive", /*RTTB_ERROR_UNEXPECTEDEOF*/
    "Out of memory", /*RTTB_ERROR_MALLOC*/
    "No such container registered", /*RTTB_ERROR_INVRTTB*/
    "No such file", /*RTTB_ERROR_NOSUCHFILE*/
    "Cannot generate a unique handle", /*RTTB_ERROR_HANDLEGEN*/
    "An unexpected error occurred", /*RTTB_ERROR_UNEXPECTED*/
    "End of file" /*RTTB_ERROR_EOF*/ /* shouldn't print this one, just handle it */
  };

  if ( ErrorNumber >= sizeof(msgs)/sizeof(msgs[0]) )
    msgptr = defaultMsg;
  else
    msgptr = msgs[ErrorNumber];

  if ( PYourString )
    strncpy(PYourString, msgptr, MaxLen+1);

  return strlen(msgptr);
}
/*-------------------------------------------------------------------------*/
void rttb_GetLibVer(unsigned int* PMajor, unsigned int* PMinor, unsigned int* PPatch)
{
  if (PMajor) (*PMajor) = RTTB_LIB_VER_MAJOR;
  if (PMinor) (*PMinor) = RTTB_LIB_VER_MINOR;
  if (PPatch) (*PPatch) = RTTB_LIB_VER_PATCH;
}
/*-------------------------------------------------------------------------*/
long int rttb_LocalToGlobalOffset(RTTB_HANDLE Handle, long int LocalOffset)
{
  if ( Handle == RTTB_NULLHANDLE || !rttb_core()->IsValidHandle(Handle) ) {
    /* <-- Set last error here when it's supported */
    return -1L;
  }
  return rttb_core()->LocalToGlobalOffsetByHandle(Handle, LocalOffset);
}
/*-------------------------------------------------------------------------*/
