/*
Daybo Logic Run-Time TarBall Library And Tools
Copyright (c) 2000-2006, David Duncan Ross Palmer, Daybo Logic
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
      
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
      
    * Neither the name of the Daybo Logic nor the names of its contributors
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#define RTTB_SOURCE

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif /*HAVE_CONFIG_H*/

#ifdef HAVE_LIMITS_H
# include <limits.h>
#endif /*HAVE_LIMITS_H*/

#include <stdio.h>

#ifdef RTTB_HDRSTOP
# pragma hdrstop
#endif /*RTTB_HDRSTOP*/

#include "rttb.h"
#include "rttb_int.h"
/*-------------------------------------------------------------------------*/
/* My shortcuts */

#ifdef getendian
# undef getendian
#endif
#ifdef hiword8
# undef hiword8
#endif
#ifdef loword8
# undef loword8
#endif
#ifdef hiword16
# undef hiword16
#endif
#ifdef loword16
# undef loword16
#endif
#ifdef swap16
# undef swap16
#endif
#ifdef swap32
# undef swap32
#endif

#define getendian() rttb_DetermineByteOrder()

#define hiword8(w16) rttb_GetWord8(wthi, (w16))
#define loword8(w16) rttb_GetWord8(wtlo, (w16))

#define hiword16(w32) rttb_GetWord16(wthi, (w32))
#define loword16(w32) rttb_GetWord16(wtlo, (w32))

#define swap16(w16) rttb_Swap16((w16))
#define swap32(w32) rttb_Swap32((w32))
/*-------------------------------------------------------------------------*/
RTTB_WORD16 rttb_Swap16(const RTTB_WORD16 W)
{
  RTTB_WORD8 lo, hi;

  /* Yes, these are backtofront intentionally */
  lo = hiword8(W);
  hi = loword8(W);

  return rttb_MakeMachWord16(hi, lo);
}
/*-------------------------------------------------------------------------*/
RTTB_WORD32 rttb_Swap32(const RTTB_WORD32 DW)
{
  RTTB_WORD16 lo, hi;

  lo = hiword16(DW);
  hi = loword16(DW);

  return rttb_MakeMachWord32(swap16(hi), swap16(lo));
}
/*-------------------------------------------------------------------------*/
RTTB_WORD16 rttb_MakeMachWord16(const RTTB_WORD8 Hi, const RTTB_WORD8 Lo)
{
  RTTB_WORD16 madeWord = (RTTB_WORD16)0U;
#ifdef __B_ENDIAN__
  madeWord |= (RTTB_WORD16)(Lo << 8);
  madeWord |= Hi;
#else /*__L_ENDIAN__*/
  madeWord |= (RTTB_WORD16)(Hi << 8);
  madeWord |= Lo;
#endif /*__B_ENDIAN__*/
  return madeWord;
}
/*-------------------------------------------------------------------------*/
RTTB_WORD32 rttb_MakeMachWord32(const RTTB_WORD16 Hi, const RTTB_WORD16 Lo)
{
  RTTB_WORD32 madeWord = (RTTB_WORD32)0U;
#ifdef __B_ENDIAN__
  madeWord |= (Lo << 16);
  madeWord |= Hi;
#else /*__L_ENDIAN__*/
  madeWord |= (Hi << 16);
  madeWord |= Lo;
#endif /*__B_ENDIAN__*/
  return madeWord;
}
/*-------------------------------------------------------------------------*/
RTTB_WORD8 rttb_GetWord8(const enum LowOrHigh_t LowOrHigh, const RTTB_WORD16 Word16)
{
  RTTB_WORD8 gotword;
#ifdef __B_ENDIAN__
    if ( LowOrHigh == wtlo )
      gotword = (RTTB_WORD8)((Word16 >> 8) & 0xFF);
    else /* Getting hi byte */
      gotword = ((RTTB_WORD8)(Word16));
#else /* Little endian */
    if ( LowOrHigh == wtlo )
      gotword = ((RTTB_WORD8)(Word16));
    else /* Getting hi byte */
      gotword = (RTTB_WORD8)((Word16 >> 8) & 0xFF);
#endif /*__B_ENDIAN__*/
  return gotword;
}
/*-------------------------------------------------------------------------*/
RTTB_WORD16 rttb_GetWord16(const enum LowOrHigh_t LowOrHigh, const RTTB_WORD32 Word32)
{
  RTTB_WORD16 gotword;
#ifdef __B_ENDIAN__
    if ( LowOrHigh == wtlo )
      gotword = (RTTB_WORD16)((Word32 >> 16) & 0xFFFF);
    else /* Getting hi byte */
      gotword = ((RTTB_WORD16)(Word32));
#else /* Little endian */
    if ( LowOrHigh == wtlo )
      gotword = ((RTTB_WORD16)(Word32));
    else /* Getting hi byte */
      gotword = (RTTB_WORD16)((Word32 >> 16) & 0xFFFF);
#endif /*__B_ENDIAN__*/
  return gotword;
}
/*-------------------------------------------------------------------------*/
int rttb_IsBigEndian()
{
  const RTTB_WORD16 testword = 0x1234;
  const unsigned char* ptestword = (const unsigned char*)&testword;

  if ( *ptestword == 0x12 )
    return 1;
  return 0;
}
/*-------------------------------------------------------------------------*/
