/*
Daybo Logic Run-Time TarBall Library And Tools
Copyright (c) 2000-2006, David Duncan Ross Palmer, Daybo Logic
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
      
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
      
    * Neither the name of the Daybo Logic nor the names of its contributors
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/


#ifndef INC_RTTB_INT_H
#define INC_RTTB_INT_H

#ifdef __cplusplus
 extern "C" {
#endif /*__cplusplus*/
/*--------------------------------------------------------------------------*/
/*
  This header is for the internal use of the rttb library only, use outside
  is not recommended and I have added a check (below) to try to prevent it
*/
#ifndef RTTB_SOURCE
# error ("rttb_int.h is not for use outside of librttb")
#endif /*!RTTB_SOURCE*/
/*------------ types used internally: rttb --------------------------------*/

/* Bind file's related information together for fblah() calls */
typedef struct _BOUNDFILE
{
  FILE* pfs; /* Stands for Pointer to File Structure */
  char* filename;
  char* modes;
} BOUNDFILE;
/*----- end of types ------------------------------------------------------*/

/*-------------- rttb_core.c ---------------------------------------------*/
/* core functions are protected from external calling by a bit of
obscurity with this structure, obtain a pointer to the functions
with rttb_core().
*/
struct _rttb_core_entries {
  void (*Init)(void);
  void (*Uninit)(void);
  int (*IsStarted)(void);
  void (*InitBoundFile)(BOUNDFILE* p);
  RTTB_HANDLE (*GenerateHandle)(void);
  unsigned int (*AddHandle)(const RTTB_HANDLE Handle, const char* ContainerFileName, const BOUNDFILE* BoundFile);
  int (*FreeHandle)(const RTTB_HANDLE Handle);
  FILE* (*GetFSP)(const RTTB_HANDLE Handle);
  int (*RegisterContainer)(const char* ContainerFileName);
  int (*UnregisterContainer)(const char* ContainerFileName);
  RTTB_ERROR (*PrepareContainer)(const char* ContainerFileName);
  int (*IsContainerRegistered)(const char* ContainerFileName);
  int (*IsContainerPrepared)(const char* ContainerFileName);
  int (*UnprepareContainer)(const char* ContainerFileName);
  RTTB_ERROR (*PrepareFile)(const char* RTTB, const char* FileName);
  int (*CheckFileExists)(const char* RTTB, const char* FileName);
  int (*IsValidHandle)(const RTTB_HANDLE Handle);
  void (*SetFileMode)(const RTTB_HANDLE RTTBHandle, const char* Mode);
  const char* (*GetFileMode)(const RTTB_HANDLE RTTBHandle);
  RTTB_HANDLE (*GetHandle)(const char* RTTB, const char* FileName);
  RTTB_ERROR (*GetFileOffset)(const RTTB_HANDLE Handle, long int* POffset);
  RTTB_ERROR (*SetFileOffset)(const RTTB_HANDLE Handle, const long int NewOffsetRecord);
  RTTB_ERROR (*GetFileLength)(const RTTB_HANDLE Handle, long int* PLength);
  size_t (*ContainedRead)(RTTB_HANDLE Handle, size_t Size, size_t N, void* PTarget, RTTB_ERROR* PError);
  long int (*LocalToGlobalOffsetByHandle)(RTTB_HANDLE Handle, long int LocalOffset);
  RTTB_BYTE (*ReadOneByte)(RTTB_HANDLE Handle, RTTB_ERROR* PError);
  char* (*ReadString)(RTTB_HANDLE Handle, char* s, unsigned int size, RTTB_ERROR* PError);
};
/*
  In order to call _any_ core functions use rttb_core() to obtain
  a pointer to the structure of pointers to functions as seen above.
*/
inline const struct _rttb_core_entries* rttb_core(void);

/*--------- rttb_debug.c --------------------------------------------------*/
/*
  All memory inside the library must use rttb_malloc() not malloc() to
  for debugging purposes.  It works in the same way as malloc(), except
  that it will abort the program if it runs out of memory.
  The module rttb_dyn.c is defunct in NDEBUG builds.
  All files to be opened etc. should be registered with the debuggable
  file manager system below.
*/
#ifndef NDEBUG
  void* rttb_debug_malloc(size_t s, const unsigned int Line, const char* File);
# define rttb_malloc(s) rttb_debug_malloc((s), (__LINE__), (__FILE__))
#else
# define rttb_malloc(s) malloc((s))
#endif /*!NDEBUG*/
/*
  All library calls to free memory must use rttb_free() as a balance to
  rttb_malloc() only.  Never free() directly.
*/
#ifndef NDEBUG
  void rttb_debug_free(void* p);
# define rttb_free(p) rttb_debug_free((p))
#else
# define rttb_free(p) free((p))
#endif /*!NDEBUG*/

/*
  This provides debugging statistics
  DebugBlocks() returns the number of blocks allocated.
*/
#ifndef NDEBUG
  unsigned long rttb_debug_DebugBlocks(void);
# define rttb_DebugBlocks() rttb_debug_DebugBlocks()
#else
# define rttb_DebugBlocks() (0UL)
#endif /*!NDEBUG*/

/* File management debuggers, don't call directly, use macros */
#ifndef NDEBUG
  int rttb_debug_FileManagerStartup(void);
# define rttb_FileManagerStartup() rttb_debug_FileManagerStartup()
#else
# define rttb_FileManagerStartup() (1) /* Just say success */
#endif /*!NDEBUG*/

#ifndef NDEBUG
  int rttb_debug_FileManagerIsStarted(void);
# define rttb_FileManagerIsStarted() rttb_debug_FileManagerIsStarted()
#else
# define rttb_FileManagerIsStarted() (1)
#endif /*!NDEBUG*/

#ifndef NDEBUG
  void rttb_debug_FileManagerShutdown(void);
# define rttb_FileManagerShutdown() rttb_debug_FileManagerShutdown()
#else
# define rttb_FileManagerShutdown()
#endif /*!NDEBUG*/


/* All FILE* handles in the library must use rttb_open & rttb_close, not fopen
and fclose directly to avoid leaking handles.  This is all stripped out in
release versions. */
#ifndef NDEBUG
  FILE* rttb_debug_open(const char* path, const char* mode);
  int rttb_debug_close(FILE* stream);
# define rttb_open(p, m)  rttb_debug_open((p), (m))
# define rttb_close(s) rttb_debug_close((s))
#else
# define rttb_open(p, m) fopen((p), (m))
# define rttb_close(s) fclose((s))
#endif /*!NDEBUG*/

/* Print out debug messages */
#ifndef NDEBUG
  void rttb_debug_dbgmsg(const char* message);
# define DBGMSG(msg) rttb_debug_dbgmsg((msg));
#else
# define DBGMSG(msg)
#endif /*!NDEBUG*/

/* This function will dump all currently register chunks of memory into a
log file called rttbchunk.log information such as sizes, owner and the
like.  Don't try to call it in the release library, it doesn't exist for
size reasons. */
#ifndef NDEBUG
  void rttb_debug_chunkdump(void);
#endif /*!NDEBUG*/
/*-------------------------------------------------------------------------*/
#ifdef __cplusplus
 }
#endif /*__cplusplus*/

#endif /*!INC_RTTB_INT_H*/
