/*
Daybo Logic Run-Time TarBall Library And Tools
Copyright (c) 2000-2006, David Duncan Ross Palmer, Daybo Logic
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
      
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
      
    * Neither the name of the Daybo Logic nor the names of its contributors
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

/* Try to stop people who aren't part of the library including this */
#ifndef RTTB_SOURCE
# error ("rttb_ll.h is for librttb's use only!")
#endif /*!RTTB_SOURCE*/

#ifndef INC_RTTB_LL_H
#define INC_RTTB_LL_H
/*-------------------------------------------------------------------------*/
#ifdef __cplusplus
 extern "C" {
#endif /*__cplusplus*/

/* This is a linked list based on pointers, it's used for all sorts
of storage inside the library.  Functions returning int are boolean,
1 = success, 0 = failure. */
struct link_ptrs_node
{
  void* ptr;
  struct link_ptrs_node* next;
};
struct link_ptrs
{
  struct link_ptrs_node* base; /* Linked list base, do not touch */
  size_t nodeSize; /* Do not touch this from outside! */
  unsigned int count; /* Don't alter the count from outside, it is used to make the count a lot faster than following the list */
  int (*Add)(struct link_ptrs* Obj, const void* Node);
  int (*Del)(struct link_ptrs* Obj, const void* Node);
  int (*DelByPos)(struct link_ptrs* Obj, const unsigned int Pos);
  void (*DelAll)(struct link_ptrs* Obj); /* Delete's everything, can be called anytime... but MUST be called before you discard the object */
  unsigned int (*Count)(const struct link_ptrs* Obj);
  int (*Find)(struct link_ptrs* Obj, const void* Node, unsigned int* PPos); /* Finds the node, PPos is optional if you want to recover the position */
  void (*Set)(struct link_ptrs* Obj, const unsigned int Pos, const void* Node);
  void (*Get)(const struct link_ptrs* Obj, const unsigned int Pos, void* Node);
  size_t (*NodeSize)(const struct link_ptrs* Obj);
};
void rttb_link_ptrs_init(struct link_ptrs* Obj, const size_t NodeSize); /* Must be used before object is used, nodes are fixed size > 0 */

#ifdef __cplusplus
 }
#endif /*__cplusplus*/
/*-------------------------------------------------------------------------*/
#endif /*!INC_RTTB_LL_H*/
