/*
Daybo Logic Run-Time TarBall Library And Tools
Copyright (c) 2000-2006, David Duncan Ross Palmer, Daybo Logic
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
      
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
      
    * Neither the name of the Daybo Logic nor the names of its contributors
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef INC_RTTB_STRZZ_H
#define INC_RTTB_STRZZ_H
#ifdef __cplusplus
 extern "C" {
# pragma message ("strzz would ideally be a class in C++, consider this advice for a good program design")
#endif /*__cplusplus*/
/*--------------------------------------------------------------------------*/
struct strzz
{
  /* Private data */
  struct link_ptrs _ll; /* Linked list containing real data */
  char* _pCreatedString; /* Created string, do not grab directly */
  unsigned int _cached; /* Wicked cache to stop re-creating the string if it's still valid */
  /* Public functions */
  void (*Add)(struct strzz*, const char* NewStr); /* Add a new string on the end */
  void (*Remove)(struct strzz*, const int Index); /* Remove a string by index */
  unsigned int (*Count)(const struct strzz*); /* Counts number of strings there are */
  int (*Find)(const struct strzz*, const char* CompareString); /* Returns index of a string -1 if not present */
  const char* (*GetByIndex)(const struct strzz*, const int Index); /* Returns a pointer to a specific string, it would NOT be wise to modify this pointer */
  void (*SetByIndex)(struct strzz*, const int Index, const char* NewString); /* Sets a string, repleaces and old one, by index */
  unsigned int (*FullLength)(const struct strzz*); /* Returns lengths of all strings together, if you want NULL terminators too add Count() */
  void (*Destruct)(struct strzz*); /* Alternative way to call destructor for object */
  const char* (*GetCString)(struct strzz*); /* Returns the NULL terminated string */
};
typedef struct strzz strzz_t; /* Alternative for if you want to use it as a type not a struct */
/*
  Specifies a user function to install if we run out of memory,
  applied to all objects, pointer to specific object is passed to
  handler, if NULL (default), a default handler is used.
  Caution as my handler does, your handler MUST destruct the object
  to make sure it's all clean if you plan to abort, otherwise just
  carray on as normal and see if you can recover from this out of
  memory error.
*/
void rttb_internal_strzzSetAbortHandler(void (*NewHandler)(struct strzz*));
/* Constructor & destructor */
void rttb_internal_strzzConstruct(struct strzz*);
void rttb_internal_strzzDestruct(struct strzz*);
/*--------------------------------------------------------------------------*/
#ifdef __cplusplus
}
#endif /*__cplusplus*/
#endif /*!INC_RTTB_STRZZ_H*/
