/*
Daybo Logic Run-Time TarBall Library And Tools
Copyright (c) 2000-2006, David Duncan Ross Palmer, Daybo Logic
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
      
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
      
    * Neither the name of the Daybo Logic nor the names of its contributors
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

/*
  Stupid rttb library user test app for personal use,
  this file is one large testing hack, the code below is a right mess
  don't do the same things I get up to here, some are short-sighted
  and very dangerous.  The gotos are just messy.
*/

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif /*HAVE_CONFIG_H*/

#include <cstdio>

#ifdef HAVE_STDLIB_H
# include <cstdlib>
#endif /*HAVE_STDLIB_H*/

#ifdef HAVE_SIGNAL_H
# include <signal>
#endif /*HAVE_SIGNAL_H*/

#ifdef HAVE_LIMITS_H
# include <limits>
#endif /*HAVE_LIMITS_H*/

#ifdef __STDC__
# include <iostream>
#else
# include <iostream.h> /* Older non-ANSI header name */
#endif /*__STDC__*/

#ifdef RTTB_HDRSTOP
# pragma hdrstop
#endif /*RTTB_HDRSTOP*/

#include "rttb.h" /* Lib's header */

#define RTTB_SOURCE /* Hack */
#define HACKED
#ifdef HACKED
# include "rttb_int.h"
# include "rttb_ll.h"
#endif
/*-------------------------------------------------------------------------*/
/* These will be used if not specified on command line */
#define DEFAULT_CONTAINER "binary.rttb"
#define DEFAULT_FILE "rttb_core.o"

typedef unsigned int UINT;
using namespace std;
/*-------------------------------------------------------------------------*/
static void TestError(const char* ErrMsg);
static void PrintTheError(RTTB_ERROR rttberror);
#ifdef HACKED
  static UINT LLTest(void);
#endif /*HACKED*/

static UINT EndianTest(void);
#ifdef __cplusplus
    static void PrintEndians(const RTTB_WORD16, const RTTB_WORD32);
#else
#   define PrintEndians(w16,w32) printf("Can\'t print endians, compile with C++\n");
#endif /*__cplusplus*/

/* This bit nabbed from a Borland help example */
#ifdef __cplusplus
   typedef void (*fptr)(int);
#else
   typedef void (*fptr)();
#endif
static void InstallSignals(void);
void Catcher(int signum); /* Do not call directly */
const char* GetName(void); /* Get a container, then a filename call twice, no more */
FILE* output = NULL; /* Nothing to do with RTTB really, a standard FILE handle */
static const char** __GlobalArgV;
static int __GlobalArgC;
/*-------------------------------------------------------------------------*/
/* In Borland C++Builder, main() is elsewhere and calls testmain()
Borland C++Builder doesn't define __BCB__ so I have done this in options |
conditional defines. */
int
#ifdef __BCB__
testmain()
#else
main(const int argc, const char* argv[])
#endif /*__BCB__*/
{
  UINT failCount;
  unsigned int maj, min, patch;
  RTTB_ERROR rttberror;
  RTTB_HANDLE handle;
  const char *containername, *filename;
  long int fl; /* File length */
  UINT i;

  __GlobalArgC = argc;
  __GlobalArgV = argv; /* Make params global */
  InstallSignals();
  printf("RTTB LIBRARY TEST - CAUTION! CAUTION! CAUTION!\n");
  printf("==============================================\n\n");

  /* Get names from commandline or use defaults */
  containername = GetName();
  filename = GetName();

  if ( !rttb_Setup() )
  {
    TestError("Couldn'\t setup librttb");
    goto messyabort;
  }
  rttb_GetLibVer(&maj, &min, &patch);
  printf("VERSION: %u.%u.%u\n", maj, min, patch);
#ifdef HACKED
  failCount = LLTest(); // Do linked list tests
  if ( failCount )
  {
    char hackThis[256];
    sprintf(hackThis, "LL: %u failures", failCount);
    TestError(hackThis);
    goto messyabort;
  }
  failCount = EndianTest(); // Do byte endian tests
  if ( failCount )
  {
    TestError("ATTENTION!  Endian swap fscker!\n");
    goto messyabort;
  }
#endif /*HACKED*/
  printf("\n\nOpening %s:%s\n", containername, filename);
  handle = rttb_fopen(containername, filename, "rb", 0, &rttberror);
  if ( handle == RTTB_NULLHANDLE ) {
    PrintTheError(rttberror);
    goto messyabort;
  }
  /* File opened successfully.  Try to dump it out to a real file to test
  library functionallty */
  if ( rttb_filelength(handle, &fl) )
    printf("Warning!   Error in getting the file length\n");
  else
    printf("%ld bytes in %s\n", fl, filename);

  printf("Attempting slow copy (byte read, byte write)\n");
  output = fopen("copytest.bin", "wb");
  if ( output ) {
    while ( !rttb_feof(handle) ) {
      unsigned char byte;

      rttb_fread(&byte, sizeof(byte), 1, handle, &rttberror);
      if ( rttberror != RTTB_ERROR_NOERROR ) {
        /* Warn the user */
        PrintTheError(rttberror);
        break;
      }
      fwrite(&byte, sizeof(byte), 1, output);
#ifndef NDEBUG
      fflush(output); /* I need this for debugging */
#endif /*!NDEBUG*/
    }
    fclose(output);
  }
  else {
    printf("Cannot open slow copy output\n");
    goto messyabort;
  }

  printf("10 or less characters dumped from %s follow:\n", filename);
  for ( i = 0U; i < 10; i++ ) {
    RTTB_ERROR blahError;
    int blah = rttb_fgetc(handle, &blahError); /* Try to get 1 char */
    if ( blahError ) {
      PrintTheError(blahError);
      break;
    }
    else {
      if ( blah == EOF ) break;
      fputc(blah, stdout);
    }
  }
  rttberror = rttb_fclose(handle);
  if ( rttberror != RTTB_ERROR_NOERROR ) {
    PrintTheError(rttberror);
    goto messyabort;
  }

  printf("Library cleanup NOW\n");
  rttb_Cleanup();
  return EXIT_SUCCESS;

messyabort:
  rttb_Cleanup();
#if defined(HACKED) && !defined(NDEBUG)
  printf("---CHUNK DUMP---\n");
  rttb_debug_chunkdump();
#endif /* HACKED && !NDEBUG */
  return EXIT_FAILURE;
}
/*-------------------------------------------------------------------------*/
static void TestError(const char* ErrMsg)
{
  fprintf(RTTB_DEV_ERROR, "test ERROR: %s\n", ErrMsg ? ErrMsg : "nothing");
}
/*-------------------------------------------------------------------------*/
static void PrintTheError(RTTB_ERROR rttberror)
{
  char* errmsg;
  unsigned int errmsglen;

  errmsglen = rttb_GetErrorMessage(rttberror, NULL, 0);
  errmsg = new char[errmsglen + 1];
  rttb_GetErrorMessage(rttberror, errmsg, errmsglen);
  TestError(errmsg);
  delete[] errmsg;
}
/*-------------------------------------------------------------------------*/
#ifdef HACKED
static UINT LLTest()
{
  UINT i, failCount = 0U;

  struct samplenode
  {
    int allo, me, just, testing, aiii;
  } instancenode[5];
  struct link_ptrs ll; /* The linked list */

  printf("\nStarting Linked list tests\n");
  rttb_link_ptrs_init(&ll, sizeof(struct samplenode));

  printf("LL: Adding\n");
  for ( i = 0U; i < sizeof(instancenode)/sizeof(instancenode[0]); i++ )
  {
    instancenode[i].allo = 0;
    instancenode[i].me = 1;
    instancenode[i].just = 2;
    instancenode[i].testing = 3;
    instancenode[i].aiii = 4;
    if ( !ll.Add(&ll, &instancenode[i]) )
    {
      TestError("LL: Whoops, add failed");
      failCount++;
    }
  }

  printf("LL: Readbacks\n");
  for ( i = 0U; i < sizeof(instancenode)/sizeof(instancenode[0]); i++ )
  {
    ll.Get(&ll, i, &instancenode[i]);
    printf("[%u]: allo=%d, me=%d, just=%d, testing=%d, aiii=%d\n",
           i,
           instancenode[i].allo,
           instancenode[i].me,
           instancenode[i].just,
           instancenode[i].testing,
           instancenode[i].aiii
    );
  }

  printf("LL: Deleting the lot\n");
  ll.DelAll(&ll);
#ifndef NDEBUG
  rttb_debug_DebugBlocks();
#endif /*!NDENUG*/
  return failCount;
}
#endif /*HACKED*/
/*-------------------------------------------------------------------------*/
#ifdef HACKED
static UINT EndianTest()
{
# define GOODREVDWORD ((RTTB_WORD32)0x78563412UL)
# define GOODREVWORD  ((RTTB_WORD16)0x3412U)
  UINT failCount = 0U;
  RTTB_WORD16 w16 = 0x1234U;
  RTTB_WORD32 w32 = (RTTB_WORD32)0x12345678UL;

  printf("Machine: %s endian\n",
#ifdef __B_ENDIAN__ /* Big endian */
    "big"
#else /* Little endian */
    "little"
#endif /*__B_ENDIAN__*/
  );
  printf("Words are normal...\n");
  PrintEndians(w16, w32);
  printf("Swapping words over...\n");
  w16 = rttb_Swap16(w16);
  w32 = rttb_Swap32(w32);
  PrintEndians(w16, w32);

  /* Do tests on endians and return # failures */
  if ( w16 != GOODREVWORD )
    failCount++;
  if ( w32 != GOODREVDWORD )
    failCount++;

  return failCount;
}
#endif /*HACKED*/
/*-------------------------------------------------------------------------*/
#if defined(__cplusplus)
static void PrintEndians(const RTTB_WORD16 w16, const RTTB_WORD32 w32)
{
  cout << "word16: " << hex << w16 << "  ";
  cout << "word32: " << hex << w32 << "  ";
  cout << endl;
}
#endif /*__cplusplus*/
/*-------------------------------------------------------------------------*/
static void InstallSignals()
{
  if ( signal(SIGINT, (fptr)Catcher) == SIG_IGN )
    signal(SIGINT, SIG_IGN);
  if ( signal (SIGTERM, (fptr)Catcher) == SIG_IGN )
    signal(SIGTERM, SIG_IGN);
  return;
}
/*-------------------------------------------------------------------------*/
void Catcher(int signum)
{
  if ( signum == SIGTERM )
    printf("\n# Terminated #\n");
  else /*SIGINT*/
    printf("\n# Aborted by user #\n");

  if ( output ) {
    fclose(output);
    output = NULL;
  }
  rttb_Cleanup();
  return;
}
/*-------------------------------------------------------------------------*/
const char* GetName()
{
  static int callCount;
  const char* ptr = NULL;

  if ( !callCount ) { /* First call */
    if ( __GlobalArgC < 1 + 1 ) /* Not sent by user */
      ptr = DEFAULT_CONTAINER; /* Use builtin */
    else
      ptr = __GlobalArgV[1];
  }
  else if ( callCount == 1 ) { /* Second call */
    if ( __GlobalArgC < 2 + 1 ) /* Not sent by user */
      ptr = DEFAULT_FILE;
    else
      ptr = __GlobalArgV[2];
  }

  if ( ptr ) /* Set OK? */
    callCount++; /* This logic prevents abuse with multiple calls (wrapping) */
  return ptr;
}
/*-------------------------------------------------------------------------*/
