/*
Daybo Logic Run-Time TarBall Library And Tools
Copyright (c) 2000-2006, David Duncan Ross Palmer, Daybo Logic
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
      
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
      
    * Neither the name of the Daybo Logic nor the names of its contributors
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

/*
  Helper functions for my linked list of source files
*/

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif /*HAVE_CONFIG_H*/

#ifdef HAVE_STDDEF_H
# include <stddef.h>
#endif /*HAVE_STDDEF_H*/

#ifdef HAVE_LIMITS_H
# include <limits.h>
#endif /*HAVE_LIMITS_H*/

#ifdef RTTB_HDRSTOP
# pragma hdrstop
#endif /*RTTB_HDRSTOP*/

#include "ddrptype.h" /* DDRP's types in the game */
#include "simplemm.h"
#include "inputll.h"

static S_SOURCE* _pBaseSources = NULL;
static unsigned int _count = 0U; /* Fast count of items in LL */

BOOL LL_AddSource(S_SOURCE* NewSourceFile)
{
  S_SOURCE* Pcurrent = _pBaseSources;

  if ( !_pBaseSources ) /* Nothing there already */
  {
    _pBaseSources = (S_SOURCE*)ddrpmalloc(sizeof(S_SOURCE));
    Pcurrent = _pBaseSources;
    if ( !_pBaseSources ) /* Allocation failure */
      return FALSE;
  }
  else /* Already have a list of filenames, transend until PNext is NULL */
  {
    while (Pcurrent->PNext)
      Pcurrent = Pcurrent->PNext;

    /* Now reached linked list entry with NULL next pointer, allocate new entry here */
    Pcurrent->PNext = ddrpmalloc(sizeof(S_SOURCE));
    Pcurrent = Pcurrent->PNext; /* Move to our fresh empty item */
    if ( !Pcurrent ) /* Allocation failed for new item */
      return FALSE;
  }

  /* Now copy data into fresh item, note fresh item is Pcurrent now */
  Pcurrent->FileName = NewSourceFile->FileName;
  Pcurrent->Length = NewSourceFile->Length;
  Pcurrent->FileNameDynamic = NewSourceFile->FileNameDynamic;
  Pcurrent->PNext = NULL; /* Last time in list */

  _count++; /* Update cached count */
  return TRUE; /* Success */
}

BOOL LL_DeleteSources()
{
  S_SOURCE* Pcurrent = _pBaseSources;
  unsigned int releaseCount = 0U;

  while ( Pcurrent )
  {
    S_SOURCE* Pderef = Pcurrent->PNext; /* Take address of next */

    /* Free the filename only if I have to */
    if ( Pcurrent->FileNameDynamic && Pcurrent->FileName )
      ddrpfree( Pcurrent->FileName );

    ddrpfree( Pcurrent ); /* Release the present item */
    releaseCount++;
    Pcurrent = Pderef; /* Get ready to release next item */
  }
  _count -= releaseCount; /* Update cached count */
  return releaseCount; /* Let caller know how many were released */
}

unsigned int LL_CountItems()
{
  return _count; /* At least we didn't have to crawl */
}

S_SOURCE* LL_Address()
{
  return _pBaseSources;
}
